
var_test_spatial <- function(dataset,
                             target_var_name,
                             weights,
                             log10_transform = F,
                             local_moran_signif = 0.05,
                             norm_plots = F) {
  
  
  ## check if log_transform option is set to TRUE
  if ( log10_transform == T ) { ## if TRUE
    dataset[,target_var_name] <- log10(dataset %>% pull(target_var_name) +1)
    
    dataset <- dataset %>%
      mutate(target_var_scaled = as.vector(scale( !!rlang::sym(target_var_name) )) )
  }
  
  if ( log10_transform == F ) { ## if FALSE
    dataset <- dataset %>%
      mutate(target_var_scaled = as.vector(scale( !!rlang::sym(target_var_name )) ) )
  }
  
  
  
  dataset <- dataset %>%
    mutate(target_var_scaled_lag = lag.listw(x = weights, var = target_var_scaled))
  
  
  moran_permutation_test <- moran.mc(dataset %>% pull(target_var_scaled),
                                     listw = weights, nsim = 999)
  

  
  gg_moran <- dataset %>%
    ggplot() +
    aes(x = target_var_scaled, y = target_var_scaled_lag) +
    geom_point(shape = 1, alpha = 0.5) +
    geom_hline(yintercept = mean(dataset %>% pull(target_var_scaled)), lty=2) +
    geom_vline(xintercept = mean(dataset %>% pull(target_var_scaled_lag)), lty=2) +
    geom_smooth(method = "lm") +
    labs(title = paste0("Variable: ", target_var_name),
         subtitle = paste0(" Moran I: ", round(moran_permutation_test$statistic, 4),
                           " p: ", round(moran_permutation_test$p.value, 4))) +
    theme_pubclean(base_size = 10) +
    coord_cartesian(clip = "off")
  gg_moran
  
  local_moran <- localmoran(x = dataset %>% pull(target_var_scaled), listw = weights)
  
  dataset <- dataset %>%
    mutate(target_var_signif = local_moran[,5]) # get the 5th column of the matrix - the `Pr(z > 0)` column
  
  
  dataset <- dataset %>% mutate(target_var_cluster = case_when(target_var_signif > local_moran_signif ~ "non-significant",
                                                               target_var_scaled > 0 & target_var_scaled_lag > 0 & target_var_signif <= local_moran_signif ~ "high-high",
                                                               target_var_scaled < 0 & target_var_scaled_lag < 0 & target_var_signif <= local_moran_signif ~ "low-low",
                                                               target_var_scaled > 0 & target_var_scaled_lag < 0 & target_var_signif <= local_moran_signif ~ "high-low",
                                                               target_var_scaled < 0 & target_var_scaled_lag > 0 & target_var_signif <= local_moran_signif ~ "low-high"))
  
  
  local_moran_map <- dataset %>%
    ggplot() +
    aes(fill = target_var_cluster) +
    geom_sf(lwd = 0.2) +
    scale_fill_manual(values = c("low-low" = "blue", "high-high" = "red", "low-high" = "dodgerblue1", "high-low" = "salmon1", "non-significant" = "grey90")) +
    labs(fill = "Cluster") +
    theme_pubclean() +
    theme(legend.position = "right")
  local_moran_map
  
  
  
  ## perform normality test
  ks_test <- ks.test(dataset %>% pull(target_var_name), y = "pnorm")
  
  
  ## set axis limits vairable to apply to all plots
  axis_lim <- range(dataset %>% pull(target_var_name))
  
  ## create historgram
  x_hist <- dataset %>%
    ggplot() + ## create ggplot object
    aes_string(x = target_var_name) + ## set aesthetics to x, our numeric vector
    geom_histogram() + ## plot histogram
    xlim(axis_lim) + ## set limits of the x axis to the predefined variable value
    theme_pubclean() ## apply theme to the plot
  
  
  ## create qqnorm
  x_qqnorm <- dataset %>%
    ggplot() +
    stat_qq(aes_string(sample = target_var_name)) +
    geom_abline(intercept = mean(dataset %>% pull(target_var_name), na.rm = T),
                slope = sd(dataset %>% pull(target_var_name), na.rm = T)) +
    ylim(axis_lim) +
    labs(title = paste0 ("Distribution of ",
                         target_var_name,
                         "\nNormality test p-value = ",
                         round(ks_test$p.value, 3),
                         ifelse(log10_transform == T, "\nLog-transformed", "\nNot log-transformed"))) +
    theme_pubclean()
  
  if(norm_plots == F){
    final_plot <- (gg_moran + local_moran_map)
  } else {
    final_plot <- (gg_moran + local_moran_map) / (x_hist + x_qqnorm)
  }
  
  return(final_plot)
}
